@@ -15,13 +15,15 @@ import threading
 from datetime import datetime
 from github import Github
 from botocore.exceptions import WaiterError
+
 logging.basicConfig(level=logging.INFO)
 
 
 # exception hook so errors raise to main thread
 def exception_hook(args):
     traceback.print_exc()
-    sys.exit(1)
+    globals()['_call_on_all_errors'] = True
+
 
 # set the exception hook
 threading.excepthook = exception_hook
@@ -148,21 +150,6 @@ class EC2Manager:
             instance_id = self._get_instance_id(name, try_once, attempts + 5)
         return instance_id
 
-    def get_terraform_outputs(self):
-        """
-        Gets the output from terraform state.
-
-        :returns: Terraform outputs.
-        :rtype: dict
-        """
-        self._logger.info(f'Exporting Terraform outputs...')
-        output = subprocess.check_output(
-            ['terraform', 'output', '-json'],
-            env=os.environ,
-            cwd=self._terraform_directory
-        )
-        return json.loads(output)
-
     def _get_instance_names(self):
         """
         Gets a list of instance names whose instances should be created.
@@ -204,7 +191,8 @@ class EC2Manager:
         return {name: {
             'commands': {
                 'start': instances[name].get('commands', {}).get('start', 'docker-compose up --detach'),
-                'stop': instances[name].get('commands', {}).get('stop', 'docker-compose down')
+                'stop': instances[name].get('commands', {}).get('stop', 'docker-compose down'),
+                'logs': instances[name].get('commands', {}).get('logs', 'docker-compose logs')
             },
             'volume_size': instances[name].get('volume_size', 8),
             'instance_type': instances[name].get('instance_type', 't4g.nano'),
@@ -315,6 +303,8 @@ class EC2Manager:
         :param bool join: Whether or not to join the threads.
         :return list[Thread]: A list of threads.
         """
+        globals()['_call_on_all_errors'] = False
+
         if not extra_args:
             extra_args = []
 
@@ -336,7 +326,7 @@ class EC2Manager:
             thread = threading.Thread(
                 name=name,
                 target=callable_instance,
-                args=args+extra_args,
+                args=args + extra_args,
                 kwargs=kwargs
             )
             thread.start()
@@ -347,6 +337,11 @@ class EC2Manager:
             for thread in threads:
                 thread.join()
             return []
+
+        # stop the main process if a thread created an error
+        if globals()['_call_on_all_errors']:
+            sys.exit(1)
+
         return threads
 
     def _await_instances(self):
@@ -359,12 +354,38 @@ class EC2Manager:
             extra_args=['instance_status_ok']
         )
 
+    def get_terraform_outputs(self):
+        """
+        Gets the output from terraform state.
+
+        :returns: Terraform outputs.
+        :rtype: dict
+        """
+        self._logger.info(f'Exporting Terraform outputs...')
+        output = subprocess.check_output(
+            ['terraform', 'output', '-json'],
+            env=os.environ,
+            cwd=self._terraform_directory
+        )
+        return json.loads(output)
+
     def get_repo(self):
         """
         Gets the repo instance.
         """
         return self._github_client.get_repo(full_name_or_id=f'{self._github_user}/{self._repo}')
 
+    def get_docker_logs(self, name, data):
+        """
+        Gets the log output from the docker container.
+        """
+        self._logger.info(f'{name} getting logs...')
+        working_directory = self._get_working_directory(data['working_directory'])
+        self.run_command(name, [
+            f'cd {working_directory} ',
+            data['commands']['logs']
+        ])
+
     def list_instances(self):
         """
         List all ec2 instances.
